/*
  gcc fb2.c -lgsl -lblas -lm  on orange
  FB m=2, monte carlo integrator.
  Manual: doccentral 
  ~taka/t/gsl-ref.pdf, page 273. Monte Carlo Integration.
  ~taka/t/gsl-1*  (source codes of the GSL). 
 */
#include <stdlib.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_monte.h>
#include <gsl/gsl_monte_plain.h>
#include <gsl/gsl_monte_miser.h>
#include <gsl/gsl_monte_vegas.h>
/* Computation of the integral,
 */

#define M 3
#define IR 1.0
double R = IR;
double exact = (4*IR*IR*M_PI);
double X[M][M]; /* i>j are not used */
double Y[M];
int Test = 1;
double Two = 2.0;

/* Fisher-Bingham integral */
double
g (double *k, size_t dim, void *params)
{
  extern double X[M][M];
  extern double Y[M];
  extern double R;
  extern double Two;
  double t[M],jac,p;
  int i,j;
  t[1-1] = R*cos(k[1-1])*cos(k[2-1]);
  t[2-1] = R*sin(k[1-1])*cos(k[2-1]);
  t[3-1] = R            *sin(k[2-1]);
  jac = R*R*cos(k[2-1]);
  p = 0.0;
  for (i=0; i<M; i++)
	for (j=i+1; j<M; j++) {
	  p += Two*t[i]*t[j]*X[i][j]; /* 2 times! */
	}
  for (i=0; i<M; i++)
	p += t[i]*t[i]*X[i][i]+t[i]*Y[i];
  p = exp(p);
  return(p*jac);
}

void
display_results (char *title, double result, double error)
{
  extern int Test;
  printf ("%s ==================\n", title);
  printf ("result = % .6f\n", result);
  printf ("sigma = % .6f\n", error);
  if (Test) {
    printf ("exact = % .6f (when X=Y=0 and R=1)\n", exact);
    printf ("error = % .6f = %.1g sigma\n", result - exact,
			fabs (result - exact) / error);
  }
}
void displayXY() {
  int i,j;
  extern double X[M][M];
  extern double Y[M];
  extern double Two;
  printf("sum(%lf*x_ij*t_i*t_j;i<j)+sum(x_ii*t_i^2)+sum(y_i*t_i)\nX=\n",Two);
  for (i=0; i<M; i++) {
	for (j=0; j<i; j++) printf("       ");   
	for (j=i; j<M; j++) {
	  printf("%3.3lf, ",X[i][j]);
	}
	printf("\n\n");
  }
  printf("Y=\n");
  for (i=0; i<M; i++) {
	printf("%3.3lf, ",Y[i]);
  }
  printf("\n\n");
}

int
main (int argc, char *argv[])
{
  double res, err;
  double xl[M-1] = { 0,     -M_PI/2.0 };
  double xu[M-1] = { 2*M_PI, M_PI/2.0 };
  const gsl_rng_type *T;
  int i,j,k;
  extern double Two;
  extern int Test;

  for (i=0; i<M; i++) for (j=0; j<M; j++) {X[i][j]=0.0; Y[i]=0.0;}

#ifdef TEST1
  /* sei14.ml test1(). Output should be 190.63 */
  X[0][0] =1.1; X[0][1] = 1.2/2; X[0][2] = 1.3/2;
  X[1][1] = 2.2; X[1][2] = 2.3/2;
  X[2][2] = 3.3;
  Y[0] = 0.1; Y[1] = 0.2; Y[2] = 0.3;
  printf("Answer should be 190.63\n");
#endif
  for (k=1; k<argc; k++) {
	if (strcmp(argv[k],"--1")==0) {
	  Two = 1.0;
	}else if (strcmp(argv[k],"--2")==0) {
	  Two = 2.0;
	}else if (strcmp(argv[k],"--help")==0) {
	  usage(); return(0);
	}else if (strncmp(argv[k],"--",2)==0) {
	  fprintf(stderr,"Unknown option\n"); usage(); return(-1);
	}else{
	  Test = 0;
	  for (i=0; i<M; i++) {
		for (j=i; j<M; j++) {
		  sscanf(argv[k],"%lf",&(X[i][j]));
		  k++;
		  if (k >= argc) break;
		}
		if (k>= argc) break;
	  }
	  for (i=0; i<M; i++) {
		sscanf(argv[k],"%lf",&(Y[i]));
		k++;
		if (k >= argc) break;
	  }
	}
  }
  
  gsl_rng *r;
  gsl_monte_function G = { &g, M-1, 0 };
  size_t calls = 500000;
  gsl_rng_env_setup ();
  T = gsl_rng_default;
  r = gsl_rng_alloc (T);
  {
	if (!Test) displayXY();
    gsl_monte_plain_state *s = gsl_monte_plain_alloc (M-1);
    gsl_monte_plain_integrate (&G, xl, xu, M-1, calls, r, s,
			       &res, &err);
    gsl_monte_plain_free (s);
    display_results ("plain", res, err);
  }
  return 0;
}

usage() {
  fprintf(stderr,"fb2 [--help] [--1 | --2] [x11 x12 x13 x22 x23 x33 y1 y2 y3]\n");
}
