/*
  gcc fb3d.c -lgsl -lblas -lm  on orange
  FB m=2, monte carlo integrator with derivatives.
  cf. fb2.c
 */
/* example:
  ./a.out --1 1.1 1.2 1.3 1.4 2.2 2.3 2.4 3.3 3.4 4.4 0.1 0.2 0.3 0.4
*/
#include <stdlib.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_monte.h>
#include <gsl/gsl_monte_plain.h>
#include <gsl/gsl_monte_miser.h>
#include <gsl/gsl_monte_vegas.h>
/* Computation of the integral,
 */

#define M 4
#define IR 1.0
double R = IR;
double exact = (2*IR*IR*IR*M_PI*M_PI);  
/* n=M, r^(n-1)*n*pi^(n/2)/gamma(n/2+1) */
double X[M][M]; /* i>j are not used */
double Y[M];
int Test = 1;
double Two = 2.0;

/* Fisher-Bingham integral */
#define fb(mm)   extern double X[mm][mm]; \
  extern double Y[mm]; \
  extern double R; \
  extern double Two; \
  double t[mm],jac,p; \
  int i,j; \
  t[1-1] = R*cos(k[1-1])*cos(k[2-1])*cos(k[3-1]); \
  t[2-1] = R*sin(k[1-1])*cos(k[2-1])*cos(k[3-1]); \
  t[3-1] = R            *sin(k[2-1])*cos(k[3-1]); \
  t[4-1] = R                        *sin(k[3-1]); \
  jac = R*R*R*cos(k[2-1])*cos(k[3-1])*cos(k[3-1]); \
  p = 0.0; \
  for (i=0; i<mm; i++) \
	for (j=i+1; j<mm; j++) { \
	  p += Two*t[i]*t[j]*X[i][j]; \
	} \
  for (i=0; i<mm; i++) \
	p += t[i]*t[i]*X[i][i]+t[i]*Y[i]; \
  p = exp(p); 

/* 1, dy1,dy2,dy3, dy4,dy1^2,dy2^2,dy3^2 */
double g0(double *k, size_t dim, void *params){fb(M); return(p*jac);}
double g1(double *k, size_t dim, void *params){fb(M); return(t[1-1]*p*jac);}
double g2(double *k, size_t dim, void *params){fb(M); return(t[2-1]*p*jac);}
double g3(double *k, size_t dim, void *params){fb(M); return(t[3-1]*p*jac);}
double g4(double *k, size_t dim, void *params){fb(M); return(t[4-1]*p*jac);}
double g11(double *k, size_t dim, void *params){fb(M); return(t[1-1]*t[1-1]*p*jac);}
double g22(double *k, size_t dim, void *params){fb(M); return(t[2-1]*t[2-1]*p*jac);}
double g33(double *k, size_t dim, void *params){fb(M); return(t[3-1]*t[3-1]*p*jac);}

void
display_results (char *title, double result, double error)
{
  extern int Test;
  printf ("%s ==================\n", title);
  printf ("result = % .6f\n", result);
  printf ("sigma = % .6f\n", error);
  if (Test) {
    printf ("exact = % .6f (when X=Y=0 and R=1)\n", exact);
    printf ("error = % .6f = %.1g sigma\n", result - exact,
			fabs (result - exact) / error);
  }
}
void displayXY() {
  int i,j;
  extern double X[M][M];
  extern double Y[M];
  extern double Two;
  printf("sum(%lf*x_ij*t_i*t_j;i<j)+sum(x_ii*t_i^2)+sum(y_i*t_i)\nX=\n",Two);
  for (i=0; i<M; i++) {
	for (j=0; j<i; j++) printf("       ");   
	for (j=i; j<M; j++) {
	  printf("%3.3lf, ",X[i][j]);
	}
	printf("\n\n");
  }
  printf("Y=\n");
  for (i=0; i<M; i++) {
	printf("%3.3lf, ",Y[i]);
  }
  printf("\n\n");
}

int
main (int argc, char *argv[])
{
  double res, err;
  double xl[M-1] = { 0,     -M_PI/2.0,-M_PI/2.0 };
  double xu[M-1] = { 2*M_PI, M_PI/2.0, M_PI/2.0 };
  const gsl_rng_type *T;
  int i,j,k,ii;
  extern double Two;
  extern int Test;
  gsl_rng *r;
  gsl_monte_function G = { &g0, M-1, 0 };
  size_t calls = 500000;
  FILE *fp;
  double resultAll[2*M+2];

  for (i=0; i<M; i++) for (j=0; j<M; j++) {X[i][j]=0.0; Y[i]=0.0;}

  for (k=1; k<argc; k++) {
	if (strcmp(argv[k],"--1")==0) {
	  Two = 1.0;
	}else if (strcmp(argv[k],"--2")==0) {
	  Two = 2.0;
	}else if (strcmp(argv[k],"--help")==0) {
	  usage(); return(0);
	}else if (strncmp(argv[k],"--",2)==0) {
	  fprintf(stderr,"Unknown option\n"); usage(); return(-1);
	}else{
	  Test = 0;
	  for (i=0; i<M; i++) {
		for (j=i; j<M; j++) {
		  sscanf(argv[k],"%lf",&(X[i][j]));
		  k++;
		  if (k >= argc) break;
		}
		if (k>= argc) break;
	  }
	  for (i=0; i<M; i++) {
		sscanf(argv[k],"%lf",&(Y[i]));
		k++;
		if (k >= argc) break;
	  }
	}
  }
  if (!Test) displayXY();
  

  fp = fopen("tmp-out-fb3d.txt","w");
  if (fp == NULL) {
	fprintf(stderr,"File open error.\n"); return(-1);
  }

#define mint(mesg) \
  G.dim = M-1; \
  gsl_rng_env_setup (); \
  T = gsl_rng_default; \
  r = gsl_rng_alloc (T); \
  { \
    gsl_monte_plain_state *s = gsl_monte_plain_alloc (M-1); \
    gsl_monte_plain_integrate (&G, xl, xu, M-1, calls, r, s, \
			       &res, &err); \
    gsl_monte_plain_free (s); \
    display_results (mesg, res, err); \
  } \
  fprintf(fp,"%lf\n",res); \
  resultAll[ii++]=res;

  ii=0;
  G.f = &g0;  mint("plain, 1"); 
  G.f = &g1;  mint("plain, dy1"); 
  G.f = &g2;  mint("plain, dy2");
  G.f = &g3;  mint("plain, dy3");
  G.f = &g4;  mint("plain, dy4");
  G.f = &g11;  mint("plain, dy1^2");
  G.f = &g22;  mint("plain, dy2^2");
  G.f = &g33;  mint("plain, dy3^2");

  fclose(fp);
  /* The result is stored in the resultAll */
  return 0;
}

usage() {
  fprintf(stderr,"fb3d [--help] [--1 | --2] [x11 x12 x13 x14 x22 x23 x24 x33 x34 x44 y1 y2 y3 y4]\n");
}
