/*
  load("fbmd.rr"); genCode(4);
  gcc fbmd.c -lgsl -lblas -lm  on orange  is equivalent to fb3d.c 
   (compile)
  FB, monte carlo integrator with derivatives.
 */
#include <stdlib.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_monte.h>
#include <gsl/gsl_monte_plain.h>
#include <gsl/gsl_monte_miser.h>
#include <gsl/gsl_monte_vegas.h>
/* Computation of the integral,
 */

#include "tmp-fbmd.h"

void
display_results (char *title, double result, double error)
{
  extern int Test;
  printf ("%s ==================\n", title);
  printf ("result = % .6f\n", result);
  printf ("sigma = % .6f\n", error);
}

void displayXY() {
  int i,j;
  extern double X[M][M];
  extern double Y[M];
  extern double Two;
  printf("sum(%lf*x_ij*t_i*t_j;i<j)+sum(x_ii*t_i^2)+sum(y_i*t_i)\nX=\n",Two);
  for (i=0; i<M; i++) {
	for (j=0; j<i; j++) printf("       ");   
	for (j=i; j<M; j++) {
	  printf("%3.3lf, ",X[i][j]);
	}
	printf("\n\n");
  }
  printf("Y=\n");
  for (i=0; i<M; i++) {
	printf("%3.3lf, ",Y[i]);
  }
  printf("\n\n");
}

int
main (int argc, char *argv[])
{
  double res, err;
#include "tmp-fbmd-b.h"
  const gsl_rng_type *T;
  int i,j,k,ii;
  extern double Two;
  extern int Test;
  gsl_rng *r;
  gsl_monte_function G = { &g0, M-1, 0 };
  size_t calls = 500000;
  FILE *fp;
  double resultAll[2*M+2];
  char fname[1024];

  for (i=0; i<M; i++) for (j=0; j<M; j++) {X[i][j]=0.0; Y[i]=0.0;}

  for (k=1; k<argc; k++) {
	if (strcmp(argv[k],"--1")==0) {
	  Two = 1.0;
	}else if (strcmp(argv[k],"--2")==0) {
	  Two = 2.0;
	}else if (strcmp(argv[k],"--help")==0) {
	  usage(); return(0);
	}else if (strncmp(argv[k],"--",2)==0) {
	  fprintf(stderr,"Unknown option\n"); usage(); return(-1);
	}else{
	  Test = 0;
	  for (i=0; i<M; i++) {
		for (j=i; j<M; j++) {
		  sscanf(argv[k],"%lf",&(X[i][j]));
		  k++;
		  if (k >= argc) break;
		}
		if (k>= argc) break;
	  }
	  for (i=0; i<M; i++) {
		sscanf(argv[k],"%lf",&(Y[i]));
		k++;
		if (k >= argc) break;
	  }
	}
  }
  if (!Test) displayXY();
  

  sprintf(fname,"tmp-out-fb%dd.txt",M-1);
  fp = fopen(fname,"w");
  if (fp == NULL) {
	fprintf(stderr,"File open error.\n"); return(-1);
  }

#define mint(mesg) \
  G.dim = M-1; \
  gsl_rng_env_setup (); \
  T = gsl_rng_default; \
  r = gsl_rng_alloc (T); \
  { \
    gsl_monte_plain_state *s = gsl_monte_plain_alloc (M-1); \
    gsl_monte_plain_integrate (&G, xl, xu, M-1, calls, r, s, \
			       &res, &err); \
    gsl_monte_plain_free (s); \
    display_results (mesg, res, err); \
  } \
  fprintf(fp,"%lf\n",res); \
  resultAll[ii++]=res;

  ii=0;
#include "tmp-fbmd-c.h"

  fclose(fp);
  /* The result is stored in the resultAll */
  return 0;
}

usage() {
  int i,j;
  fprintf(stderr,"tmp-fb%dd [--help] [--1 | --2] [",M-1);
  for (i=1; i<=M; i++) {
    for (j=i; j<=M; j++) {
	  fprintf(stderr,"x%d%d ",i,j);
    }
  }
  for (i=1; i<=M; i++) fprintf(stderr,"y%d ",i);
  fprintf(stderr,"]\n");
  fprintf(stderr,"The output data will be written to tmp-out-fb%d.txt\n",M-1);
}
