/* 
Example from taka_runge_kutta.rr
Airy eq 
y'' - ty = 0  
---> 
d/dt y  =  y'
d/dt y' = xy
[y(0), y'(0)] = 0.3550280540, -0.2588194038]

gcc gsl_ode_test2b.c -lgsl -lblas -lm
*/
#include <stdio.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_odeiv.h>

/*
d/dt y_i = f_i(t, y_1, ..., y_n),  f_i=f[i] $B$r(B func $B$G7W;;(B.
 y[0] = y, y[1]=y'.
*/
int func (double t, const double y[], double f[], void *params)
{
	f[0] = y[1];
	f[1] = t * y[0];
	return GSL_SUCCESS;
}

int main (void)
{
	 /* $B%"%k%4%j%:%`(B T $B$rMQ$$$F(B, $B<!85(B 2 $B$NJ}Dx<0$r2r$/%9%F%C%W4X?t$r@8@.(B */
	/* Runge-Kutta-Fehlberg(4,5) */
	const gsl_odeiv_step_type *T = gsl_odeiv_step_rkf45;
	gsl_odeiv_step *s = gsl_odeiv_step_alloc(T, 2);

	 /* 
	  * $B3F%9%F%C%W$K$*$$$F2r$NCM(B y_i(t) $B$KBP$9$k8m:9$r@dBP8m:9(B 1e-6
	  * $BAjBP8m:9(B 0.0 $B$NHO0OFb$KM^$($k$h$&$J%9%F%C%WD4@0K!$r@8@.(B
	  */
	gsl_odeiv_control *c = gsl_odeiv_control_y_new(1e-6, 0.0);

	/* 2 $B<!85$NJ}Dx<0$N$?$a$N5a2r4X?t$N@8@.(B */
	gsl_odeiv_evolve *e = gsl_odeiv_evolve_alloc(2);
	gsl_odeiv_system sys = {func, NULL, 2, NULL};
	/* t : start, t1 : goal */
	double t = 0.0, t1 = -5.0; /* $B$3$NNc$G$OIi$NJ}8~$X(B */
	double h = -1e-6; /* $B%9%F%C%WI}$N=i4|CM(B */
	double y[2] = {0.3550280540, -0.2588194038}; /* y $B$N=i4|CM(B */

	while (t > t1) {
		int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &t, t1, &h, y);
		if (status != GSL_SUCCESS) break;
		printf("%.5e %.5e %.5e\n", t, y[0], y[1]);
	}

	gsl_odeiv_evolve_free(e);
	gsl_odeiv_control_free(c);
	gsl_odeiv_step_free(s);
	return 0;
}
