/* ex1b.c $B$N(B GSL $B;HMQ(B version 
   exp(xt-t^3) 
   $BF;>l$N#6>O$NNcBj(B. $BHyJ,J}Dx<0$O2<$N$h$&$KJQ$($F$^$9(B.
   gcc gsl_ode_test3.c -lgsl -lblas -lm
*/
/*  [[p],[dt,q]] --> ( p - dt q ) f = 0.  
    (-3*dx^2+x) g - dt g = 0.
    (-3*dx^2+x) z - g(infty)+g(0)= 0.
    (-3*dx^2+x) z  = -1.  
    z0=z;
    z0' = z1,
    z1' = z0'' = (x/3)*z0 + (1/3)
    P=
    [[0,  1],
     [x/3,0]]
    exp(-x*T1+T3)*[z0,z1] $B$rHyJ,$9$k$H(B
     -T1*exp(-x*T1+T3)*[z0,z1]+exp(-x*T1+T3)*[z0,z1]'
   = -T1*exp(-x*T1+T3)*[z0,z1]+P*exp(-x*T1+T3)*[z0,z1]+exp(-x*T1+T3)*[0,1/3]
   = (-T*E + P)*exp(-x*T1+T3)*[z0,z1] + exp(-x*T1+T3)*[0,1/3]
  $B$^$H$a$F(B
   [y0,y1]' = (-T*E+P)*[y0,y1]+exp(-x*T1+T3)*[0,1/3]
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gsl/gsl_integration.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_odeiv.h>
#define TMAX 100.0
double T1;
double T3;
     
double f (double t, void * params) {
  double f = exp(-t*t*t);
  return f;
}
double fd (double t, void * params) {
  double f = t*exp(-t*t*t);
  return f;
}
     
double
f0(void)
{
  gsl_integration_workspace * w 
        = gsl_integration_workspace_alloc (1000);
       
  double result, error;
     
  gsl_function F;
  F.function = &f;

     
  gsl_integration_qags (&F, 0, TMAX, 0, 1e-7, 1000,
                                                w, &result, &error); 
     
  printf ("result          = % .18f\n", result);
  printf ("estimated error = % .18f\n", error);
  printf ("intervals =  %d\n", w->size);
     
  return result;
}

double
f1(void)
{
  gsl_integration_workspace * w 
        = gsl_integration_workspace_alloc (1000);
       
  double result, error;
     
  gsl_function F;
  F.function = &fd;

     
  gsl_integration_qags (&F, 0, TMAX, 0, 1e-7, 1000,
                                                w, &result, &error); 
     
  printf ("result          = % .18f\n", result);
  printf ("estimated error = % .18f\n", error);
  printf ("intervals =  %d\n", w->size);
     
  return result;
}

#define RANK 2

double rkmin(double x0,double y0[],double xn,double minval[]);
int rf(double x, const double *y, double *val, void *params);

main() {
  double y0[RANK];
  double minval[RANK];
  double x0,xn,minx;
  extern double T1,T3;
  T1 = 1.0; /* data average sum(t_i)/total */
  T3 = 1.0; /* data average sum(t_i^3)/total */
  x0 = 0.0; /* [x0,xn] is the search area */
  xn = 5.0;
  /* $B=i4|CM(B */
  y0[0] = f0()*exp(-0*T1+T3); 
  y0[1] = f1()*exp(-0*T1+T3);
  printf("Init y0=%lf\n", y0[0]);
  printf("Init y1=%lf\n", y0[1]);

  /* const gsl_odeiv_step_type *T = gsl_odeiv_step_rk4; */
  const gsl_odeiv_step_type *T = gsl_odeiv_step_rkf45;
  gsl_odeiv_step *s = gsl_odeiv_step_alloc(T, RANK);
  gsl_odeiv_control *c = gsl_odeiv_control_y_new(1e-6, 0.0);

  gsl_odeiv_evolve *e = gsl_odeiv_evolve_alloc(RANK);
  gsl_odeiv_system sys = {rf, NULL, RANK, NULL};
  double t = 0.0, t1 = 3.37; 
  double h = 1e-6;
  while (t < t1) {
  	int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &t, t1, &h, y0);	
	if (status != GSL_SUCCESS) break;
	printf("%.5e %.5e %.5e\n", t, y0[0], y0[1]);
  }

gsl_odeiv_evolve_free(e);
gsl_odeiv_control_free(c);
gsl_odeiv_step_free(s);
}



/* $B%Y%/%H%kCM4X?t(B f(x, y) (x : scalar, y : vector) */ 
/* val : return value                              */
/*    P=
    [[0,  1],
     [x/3,0]]
    (-T1*E + P)*[Y0,Y1] + exp(-x*T1+T3)*[0,-1/3]
     [0,1/3] $B$X(B.
*/

int rf(double x, const double *y, double *val, void *params)
{
  extern double T1;
  extern double T3;
  val[0] = -T1*y[1-1]+y[2-1];
  val[1] = (x/3)*y[1-1]-T1*y[2-1] + exp(-x*T1+T3)*(1.0/3.0);
  return GSL_SUCCESS;
}

void show_v(double *v, int n)
{
  int i;

  for (i = 0; i < n; i++)
    printf("%lf ", v[i]);
}

